using System;
using Microsoft.SharePoint;

namespace RH
{
    static class ContentTypes
    {
        /// <summary>
        /// Enumerates all Content Types from the RootWeb of a Site
        /// </summary>
        /// <param name="web"></param>
        public static void EnumContentTypes(SPWeb web)
        {
            try
            {
                Console.WriteLine("Name                Parent              Hidden              Group");
                Console.WriteLine("--------------------------------------------------------------------------------");
                SPContentTypeCollection contentTypes = web.ContentTypes;
                foreach (SPContentType type in contentTypes)
                {
                    string name = type.Name.PadRight(20);
                    string parent = type.Parent.Name.PadRight(20);
                    string hidden = type.Hidden.ToString().PadRight(20);
                    string group = type.Group;

                    Console.WriteLine(string.Format("{0}{1}{2}{3}", name, parent, hidden, group));
                }
                Console.WriteLine(string.Format("found {0} content types", contentTypes.Count));
            }
            catch (Exception ex)
            {
                Console.WriteLine(string.Format("Error enumerating the ContentTypes on {0}({1}):\r\n{2}", web.Title, web.Url, ex));
            }
            
        }

        /// <summary>
        /// Creates a new content type.
        /// if there are 2 parameters, we will try to inherit from the second one
        /// </summary>
        /// <param name="web"></param>
        /// <param name="newName">Name for the new content type</param>
        /// <param name="parentContentTypeName"></param>
        public static void CreateContentType(SPWeb web, string newName, string parentContentTypeName)
        {
            try
            {
                if (string.IsNullOrEmpty(newName))
                {
                    Console.WriteLine("Usage: SharePointConsole CreateContentType NewName [parent content type]");
                    return;
                }

                if (!string.IsNullOrEmpty(parentContentTypeName))
                {
                    SPContentType parentContentType = web.ContentTypes[parentContentTypeName];
                    if (parentContentType == null)
                    {
                        Console.WriteLine(string.Format("Parent Content Type '{0}' not found.", parentContentTypeName));
                        return;
                    }

                    var newType = new SPContentType(parentContentType, web.ContentTypes, newName);
                    web.ContentTypes.Add(newType);
                    Console.WriteLine(string.Format("content type '{0}' created. Parent content type is '{1}'", newName, parentContentTypeName));
                }
                else
                {
                    var newType = new SPContentType(null, web.ContentTypes, newName);
                    web.ContentTypes.Add(newType);
                    Console.WriteLine(string.Format("content type '{0}' created.", newName));
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(string.Format("Error creating the content type '{0}':\r\n{1}", newName, ex));
            }
        }

        /// <summary>
        /// Renames a content type
        /// </summary>
        /// <param name="web"></param>
        /// <param name="contentTypeName">Name of the content type to rename</param>
        /// <param name="newName">new name for the content type</param>
        public static void RenameContentType(SPWeb web, string contentTypeName, string newName)
        {
            try
            {
                if (string.IsNullOrEmpty(newName))
                {
                    Console.WriteLine("Please specify a new name for the content type.");
                    return;
                }

                SPContentType contentType = web.ContentTypes[contentTypeName];
                if(contentType == null)
                {
                    Console.WriteLine(string.Format("content type '{0}' not found.", contentTypeName));
                    return;
                }

                contentType.Name = newName;
                contentType.Update();
                Console.WriteLine(string.Format("content type '{0}' renamed to '{1}'", contentTypeName, newName));
            }
            catch (Exception ex)
            {
                Console.WriteLine(string.Format("Error renaming the content type '{0}' to '{1}':\r\n{2}", contentTypeName, newName, ex));
            }
        }

        /// <summary>
        /// Deletes a new content type.
        /// </summary>
        /// <param name="web"></param>
        /// <param name="contentTypeName">content type to be deleted</param>
        public static void DeleteContentType(SPWeb web, string contentTypeName)
        {
            try
            {
                if (string.IsNullOrEmpty(contentTypeName))
                {
                    Console.WriteLine("Usage: SharePointConsole DeleteContentType Name");
                    return;
                }

                SPContentType contentType = web.ContentTypes[contentTypeName];
                if(contentType == null)
                {
                    Console.WriteLine(String.Format("Content type '{0}' not found.", contentTypeName));
                    return;
                }

                web.ContentTypes.Delete(contentType.Id);
                Console.WriteLine(string.Format("Content type '{0}' deleted.", contentTypeName));
            }
            catch (Exception ex)
            {
                Console.WriteLine(string.Format("Error deleting the content type '{0}':\r\n{1}", contentTypeName, ex));
            }
        }
    }
}
